@@ -9,7 +9,7 @@
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 /*eslint-env node */
-var api = require('../api'), writeError = api.writeError;
+var api = require('../api'), writeError = api.writeError, writeResponse = api.writeResponse;
 var path = require('path');
 var git = require('nodegit');
 var express = require('express');
@@ -21,12 +21,13 @@ module.exports = {};
 
 module.exports.router = function(options) {
 	var fileRoot = options.fileRoot;
-	var contextPath = options.options.configParams["orion.context.path"] || "";
-	if (!fileRoot) { throw new Error('options.root is required'); }
+	var gitRoot = options.gitRoot;
+	if (!fileRoot) { throw new Error('options.fileRoot is required'); }
+	if (!gitRoot) { throw new Error('options.gitRoot is required'); }
 
 	return express.Router()
 	.use(bodyParser.json())
-	.get('/:refName'+ contextPath + '/file/*', getBlame);
+	.get('/:refName'+ fileRoot + "/*", getBlame);
 	
 function getBlame(req, res) {
 	var blamerepo,fileDir, fileRelativePath;
@@ -50,11 +51,11 @@ function getBlame(req, res) {
 			var sendingBlamejason;
 			sendingBlamejason = {
 				"Children" : blamejason,
-				"CloneLocation": contextPath + "/gitapi/clone" + fileDir ,
-				"Location": contextPath + "/gitapi/blame/"+ req.params.refName + fileDir + fileRelativePath,
+				"CloneLocation": gitRoot + "/clone" + fileDir ,
+				"Location": gitRoot + "/blame/"+ req.params.refName + fileDir + fileRelativePath,
 				"Type" : "Blame"
 			};
-			res.status(200).json(sendingBlamejason);
+			writeResponse(200, res, null, sendingBlamejason, true);
 		});
 	}).catch(function(err){
 		writeError(403, res, err);
